﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.UI;

public class GameController : MonoBehaviour {
	public Text output;

	public Hero testHero;

	public Camera lbCam;
	public Camera poseableCam;
	public Camera rotatingCam;
	public GameObject rotatingCamHolder;
	public Camera heroCam;
	public GameObject heroCamHolder;


	private int panelIndex = 0;
	private List<string> panelOrder;
	[HideInInspector]
	public Panel currentPanel;
	

	// Use this for initialization
	void Start () {

		// create the array to hold the panel order
		panelOrder = new List<string>();
		panelOrder.Add(Constants.PANEL_LB_ROTATE);
		panelOrder.Add(Constants.PANEL_WORLD_ROTATE);
		panelOrder.Add(Constants.PANEL_HERO_RUN);
		panelOrder.Add(Constants.PANEL_BATMAN);
		panelOrder.Add(Constants.PANEL_FLY_FORWARD);
		panelOrder.Add(Constants.PANEL_LB_ROTATE);
		panelOrder.Add(Constants.PANEL_WORLD_ROTATE);
		panelOrder.Add(Constants.PANEL_DANCE_PARTY);
		panelOrder.Add(Constants.PANEL_SLOW_WALK);
		panelOrder.Add(Constants.PANEL_BATMAN);
		panelOrder.Add(Constants.PANEL_FIGHT);
		panelOrder.Add(Constants.PANEL_FLY_OVER);

		// schedule load data initially
		TextAsset worldData= Resources.Load("WorldData") as TextAsset;
		processTextData(worldData.text);
		
		// Set a default "panel" - circling the LB building
		LoadNextPanel();
	}

	void Update() {
		if (currentPanel != null) {
			currentPanel.UpdatePanel();
		}
	}
	

	// Data Loading ------------------------------

	private void processTextData(string data) {
		
		// split return data by <key> parameter
		// -- could also parse XML
		string[] seperators = new string[] {"<key>"};
		string[] split = data.Split(seperators, System.StringSplitOptions.RemoveEmptyEntries);
		
		// 0 is overhead data
		// 1 is index

		string buildingString = getIntegerFromData(split[1]);
		BuildingManager.nextIndex = int.Parse(buildingString);
		
		string heroString = getIntegerFromData(split[2]);
		HeroManager.nextIndex = int.Parse(heroString);

		// Update Hero Check
		if (HeroManager.nextIndex == HeroManager.currentIndex+1) {
			GetComponent<HeroManager>().addNewHero();
		}
		else if (HeroManager.nextIndex > HeroManager.currentIndex) {
			GetComponent<HeroManager>().addPreviousHeroes();
		}

		// Update Building Check
		if (BuildingManager.nextIndex == BuildingManager.currentIndex+1) {
			GetComponent<BuildingManager>().addNewBuilding();
			ChangeToBuildingPanel();
		}
		else if (BuildingManager.nextIndex > BuildingManager.currentIndex) {
			GetComponent<BuildingManager>().addPreviousBuildings();
		}

	}


	// Utility Methods --------------------------------
	public string getIntegerFromData(string data) {
		string[] seperator = new string[] {"<integer>"};
		string[] split = data.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		string backHalf = split[1];
		seperator = new string[] {"</integer>"};
		split = backHalf.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		return split[0];
	}
	
	
	public string getStringFromData(string data) {
		string[] seperator = new string[] {"<string>"};
		string[] split = data.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		string backHalf = split[1];
		seperator = new string[] {"</string>"};
		split = backHalf.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		return split[0];
	}


	// Camera Controls ------------------------------
	public void ShowLBCam() {
		lbCam.enabled = true;
		poseableCam.enabled = false;
		rotatingCam.enabled = false;
		heroCam.enabled = false;
	}

	public void ShowPoseableCam() {
		lbCam.enabled = false;
		poseableCam.enabled = true;
		rotatingCam.enabled = false;
		heroCam.enabled = false;
	}

	public void ShowRotatingCam() {
		lbCam.enabled = false;
		poseableCam.enabled = false;
		rotatingCam.enabled = true;
		heroCam.enabled = false;
	}

	public void ShowHeroCam() {
		lbCam.enabled = false;
		poseableCam.enabled = false;
		rotatingCam.enabled = false;
		heroCam.enabled = true;
	}

	// Panel Controls ------------------------------
	public void ChangePanel(Panel newPanel) {
		if (currentPanel!=null) {
			currentPanel.PanelCleanup();
			currentPanel = null;
		}

		currentPanel = newPanel;
	}

	public void ChangeToBuildingPanel() {
		if (currentPanel!=null) {
			currentPanel.PanelCleanup();
			currentPanel = null;
		}

		currentPanel = new Panel_BuildingIntro();
	}

	public void StartPanelTimer() {
		if (currentPanel!=null)
			currentPanel.StartTimer();
	}

	public void LoadNextPanel() {
		if (currentPanel!=null) {
			currentPanel.PanelCleanup();
			currentPanel = null;
		}

		LoadNextPanelAction();
	}
	
	void LoadNextPanelAction() {
		switch (panelOrder[panelIndex]) {
		case Constants.PANEL_LB_ROTATE:
			currentPanel = new Panel_LBRotate();
			break;
		case Constants.PANEL_WORLD_ROTATE:
			currentPanel = new Panel_WorldRotate();
			break;
		case Constants.PANEL_HERO_INTRO:
			currentPanel = new Panel_HeroIntro();
			break;
		case Constants.PANEL_LB_PAN:
			currentPanel = new Panel_LBPanUp();
			break;
		case Constants.PANEL_BUILDING_INTRO:
			currentPanel = new Panel_BuildingIntro();
			break;
		case Constants.PANEL_FLY_OVER:
			currentPanel = new Panel_FlyOver();
			break;
		case Constants.PANEL_HERO_RUN:
			currentPanel = new Panel_HeroRun();
			break;
		case Constants.PANEL_BATMAN:
			currentPanel = new Panel_Batman();
			break;
		case Constants.PANEL_FIGHT:
			currentPanel = new Panel_Fight();
			break;
		case Constants.PANEL_SLOW_WALK:
			currentPanel = new Panel_SlowWalk();
			break;
		case Constants.PANEL_DANCE_PARTY:
			currentPanel = new Panel_DanceParty();
			break;
		case Constants.PANEL_FLY_FORWARD:
			currentPanel = new Panel_FlyForward();
			break;
		}
		
		panelIndex += 1;
		if (panelIndex == panelOrder.Count) {
			panelIndex = 0;
		}
	}                

}
