﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class BuildingManager : MonoBehaviour {
	public static Building newestBuilding;

	// indexs, from data
	public static int currentIndex = 0;
	public static int nextIndex = 0;

	private static int xVal = -2;
	private static int zVal = 1;
	private static int xOffset = 0;
	private static int zOffset = -1;
	private static float buildingOffset = 47.58f;
	public static Vector3 newBuildingLocation;

	public Building theBuilding;
	public static List<Building> buildingList;

	private bool introBuilding;

	void Start() {
		
	}

	public void addNewBuilding() {
		introBuilding = true;

		currentIndex = nextIndex;
		loadMetadata(nextIndex);
	}

	public void addPreviousBuildings() {
		introBuilding = false;

		currentIndex = 1;

		// start at beginning
		loadMetadata(currentIndex);
	}

	private void loadMetadata(int id) {
		TextAsset buildingData = Resources.Load("Building/"+id+"/BuildingData") as TextAsset;
		processTextData(buildingData.text);
	}
	
	
	private void processTextData(string data) {
		if (buildingList==null) {
			buildingList = new List<Building>();
		}
		
		BuildingData buildingData = new BuildingData();

		string[] seperators = new string[] {"<key>"};
		string[] split = data.Split(seperators, System.StringSplitOptions.RemoveEmptyEntries);
		
		buildingData.floors = getIntegerFromData(split[1]);
		buildingData.index = getIntegerFromData(split[2]);

		createBuilding(buildingData);
		
		// check to see if more building need to be loaded
		if (currentIndex != nextIndex) {
			currentIndex++;
			loadMetadata(currentIndex);
		}

	}


	private void createBuilding(BuildingData data) {
		// determine building location 
		Vector3 newBuildingLocation = new Vector3( xVal* buildingOffset, 0, zVal*buildingOffset );

		// adjust offset values - creating a spiral effect around the center building
		if (xVal==-zVal && zVal>0) {
			xVal -= 1;
			zVal += 1;
		}
		
		xVal += xOffset;
		zVal += zOffset;
		
		if (xVal==zVal && xVal<0) {
			xOffset = 1;
			zOffset = 0;
		}
		else if (xVal == -zVal && zVal<0) {
			zOffset = 1;
			xOffset = 0;
		}
		else if (xVal==zVal && xVal>0) {
			xOffset = -1;
			zOffset = 0;
		}
		else if (xVal==-zVal && zVal>0) {
			zOffset = -1;
			xOffset = 0;
		}

		Building newBuilding = Instantiate(theBuilding);
		newBuilding.buildFromData(data, newBuildingLocation);
		newBuilding.updateTextures(currentIndex);

		newestBuilding = newBuilding;
		buildingList.Add(newBuilding);

		if (introBuilding) {
			newBuilding.SendToIntroLocation();
		}
	}

	public static Building GetRandomBuilding() {
		if (buildingList != null) {
			if (buildingList.Count > 0) {
				
				int index = (int)Random.Range(0,buildingList.Count);
				return buildingList[ index ];
			}
		}
		
		return null;
	}

	public static Vector3 GetRandomIntersection() {
		if (buildingList != null) {
			if (buildingList.Count > 0) {
				
				int index = (int)Random.Range(0,buildingList.Count);
				Transform loc = buildingList[ index ].transform;
				return new Vector3 (loc.position.x + 23.66f, 0.1f, loc.position.z-23.66f);
			}
		}
		
		return new Vector3();
	}

	// Utility Methods --------------------------------
	public string getIntegerFromData(string data) {
		string[] seperator = new string[] {"<integer>"};
		string[] split = data.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		string backHalf = split[1];
		seperator = new string[] {"</integer>"};
		split = backHalf.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		return split[0];
	}
	
	
	public string getStringFromData(string data) {
		string[] seperator = new string[] {"<string>"};
		string[] split = data.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		string backHalf = split[1];
		seperator = new string[] {"</string>"};
		split = backHalf.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		return split[0];
	}

}
