﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class HeroManager : MonoBehaviour  {
	public static Hero newestHero;

	// indexs, from data
	public static int currentIndex = 0;
	public static int nextIndex = 0;

	public static List<HeroData> heroDataList;
	public static List<Hero> heroList;

	public Hero boyHero;
	public Hero girlHero;
	public Hero boyHeroLarge;
	public Hero girlHeroLarge;

	void Start () {
		
	}
	
	public void addNewHero() {
		currentIndex = nextIndex;
		loadMetadata(nextIndex);
	}

	public void addPreviousHeroes() {
		currentIndex = 1;
		
		// start at beginning
		loadMetadata(currentIndex);
	}


	public void loadMetadata(int id) {
		
		TextAsset heroData = Resources.Load("Hero/"+id+"/HeroData") as TextAsset;
		processTextData(heroData.text);
	}

	private void processTextData(string data) {

		if (heroDataList == null) {
			heroDataList = new List<HeroData>();
		}
		
		if (heroList == null) {
			heroList = new List<Hero>();
		}
		
		// create a HeroData object
		HeroData heroData = new HeroData();
		
		string[] seperators = new string[] {"<key>"};
		string[] split = data.Split(seperators, System.StringSplitOptions.RemoveEmptyEntries);

		// Gender
		heroData.gender = getStringFromData(split[1]);
		heroData.index = getIntegerFromData(split[2]);
		heroData.heroName = getStringFromData(split[3]);
		heroData.power1 = getStringFromData(split[4]);
		heroData.power2 = getStringFromData(split[5]);
		heroData.size = getStringFromData(split[6]);
		
		// add hero data to list
		heroDataList.Add(heroData);

		CreateHero(heroData);

		// check to see if more building need to be loaded
		if (currentIndex != nextIndex) {
			currentIndex++;
			loadMetadata(currentIndex);
		}
	}

	// Hero Instantiation Methods -------------------------
	private void CreateHero(HeroData data) {
		//Hero newHero;
		if (data.gender == "M") {
			if (data.size == "N") {
				newestHero = Instantiate(boyHero);
			}
			else if (data.size == "L") {
				newestHero = Instantiate(boyHeroLarge);
			}
		}
		else if (data.gender == "F") {
			if (data.size == "N") {
				newestHero = Instantiate(girlHero);
			}
			else if (data.size == "L") {
				newestHero = Instantiate(girlHeroLarge);
			}
		}

		// update textures
		newestHero.updateTextures(HeroManager.currentIndex);

		// save data
		newestHero.data = data;

		// store newly created Hero
		heroList.Add(newestHero);
	}


	// Limbo Methods ----------------
	public static void SendToLimbo(Hero limboHero) {
		limboHero.ResetLocation();
	}

	public static Hero getRandomHero() {
		if (heroList != null) {
			if (heroList.Count > 0) {
				int index = (int)Random.Range(0,heroList.Count);
				return heroList[ index ];
			}
		}

		return null;
	}

	public static List<Hero> getRandomHeroes(int numHeroes) {

		if (heroList != null) {

			// if array is smaller, simply return it
			if (numHeroes >= heroList.Count)  {
				return heroList;
			}
			else {
				List<Hero> tempHeroes = new List<Hero>(heroList);
				
				List<Hero> returnList = new List<Hero>();

				for (int i = 0; i<numHeroes; i++) {
					int index = (int)Random.Range(0, tempHeroes.Count);
					returnList.Add ( tempHeroes[index] );
					tempHeroes.RemoveAt(index);
				}

				return returnList;
			}

		}

		return null;
	}

	public static List<Hero> getHeroesByPower(string power) {
		return null;
	}


	// Utility Methods --------------------------------
	public string getIntegerFromData(string data) {
		string[] seperator = new string[] {"<integer>"};
		string[] split = data.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		string backHalf = split[1];
		seperator = new string[] {"</integer>"};
		split = backHalf.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		return split[0];
	}
	
	
	public string getStringFromData(string data) {
		string[] seperator = new string[] {"<string>"};
		string[] split = data.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		string backHalf = split[1];
		seperator = new string[] {"</string>"};
		split = backHalf.Split(seperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		return split[0];
	}
}
